package pep;

import java.io.*; import java.util.*;
import static pep.Main.nSubjects; import static pep.Main.numPerVar;
import static pep.Main.multTarg;  import static pep.Main.multDist;
import static pep.Main.multCue;

/**This class computes participant mean RT and percentage error, then prints
 * results to Means.txt. It can be run independently of the full model.*/
public class DataAnalysis{
 //----------VARIABLES--------------------------------------------------
 /**file name*/              static String fileName;
 /**number of variables*/    static int nVariables = numPerVar.length;
 //---------------------------------------------------------------------
 /**number of conditions*/   static int nConditions;
 /**condition number*/       static int condition;
 /**response time*/          static int rt;
 /**sum of response times*/  static int[] sumRT;
 /**number correct*/         static int[] nRT;
 /**mean response time*/     static int[] xRT;
 /**accuracy*/               static double accuracy;
 /**sum of accuracy scores*/ static double[] sumACC;
 /**number of responses*/    static int nACC[];
 /**percentage error*/       static double[] xERR;
 //-------------------------------------------------------------------
 /**check sequence?*/        static boolean isCheckSeq = Main.isCheckSeq;
 /**filter repetitions?*/    static boolean isTrimmed = Main.isTrimmed;
 /**check all rep types*/    static boolean isAllSeq = Main.isAllSeq;
 /**cued task switching?*/   static boolean isCueSwitch = Main.isCueSwitch;
 //--------------------------------------------------------------------
 /**distracter number*/      static int[] dist = new int[multDist];
 /**previous distracter*/    static int n1dist = 999;
 /**target number*/          static int[] targ = new int[multTarg];
 /**previous target*/        static int n1targ = 999;
 /**response number*/        static int resp = 999;
 /**previous response*/      static int n1resp = 999;
 /**cue number*/             static int[] cue = new int[multCue];
 /**previous cue*/           static int n1cue = 999;
 /**task number*/            static int task = 999;
 /**previous task*/          static int n1task = 999;
 /**congruency*/             static int cong = 999;
 /**previous congruency*/    static int n1cong = 999;

 /**Clears arrays and calls getMeans() for each participant.*/
 public static void main(String[] args){
  for(int c=0; c<multCue; c++) cue[c] = 999;
  for(int t=0; t<multTarg; t++) targ[t] = 999;
  for(int d=0; d<multDist; d++) dist[d] = 999;
  nConditions = 1;
  for(int k=0; k<nVariables; k++){
   nConditions *= (numPerVar[k]);
  }
  if(isAllSeq) nConditions = 15;
  else if(isCheckSeq) nConditions*=2;
  else if(isCueSwitch) nConditions = 10;
  for(int i=1; i<=nSubjects; i++){
   for(int v=0; v<nVariables; v++){
    sumRT = new int[nConditions];  sumACC = new double[nConditions];
    nRT = new int[nConditions];    nACC = new int[nConditions];
    xRT = new int[nConditions];    xERR = new double[nConditions];
   }
   getMeans(i);
  }
 }

 /**Opens a participant datafile and calculates their condition means.
  * @param i Participant number, used for opening datafile.*/
 public static void getMeans(int i){
  BufferedReader in = null;
  StringTokenizer tokenizer = null;
  try{
   fileName = "out"+i+".txt";
   in = new BufferedReader(new FileReader(fileName));
  }
  catch(Exception e){
   System.out.println("Error setting input stream.");
  }
  try{
   tokenizer = new StringTokenizer(in.readLine());
  }
  catch(Exception e){
   System.out.println("No data in file.");
  }
  while(in!=null){
   try{
    condition=0;
    tokenizer.nextToken(); tokenizer.nextToken(); //not relevant
    for(int c=0; c<multCue; c++)
     cue[c] = Integer.parseInt(tokenizer.nextToken());
    for(int t=0; t<multTarg; t++)
     targ[t] = Integer.parseInt(tokenizer.nextToken());
    for(int d=0; d<multDist; d++)
     dist[d] = Integer.parseInt(tokenizer.nextToken());
    int orthogCount = 1;
    for(int n=0; n<nVariables; n++){
     condition += (Integer.parseInt(tokenizer.nextToken())*orthogCount);
     orthogCount *= (numPerVar[n]);
    }
    task = (int)((condition) / 2);
    cong = condition;
    if(n1cong<999 & isCheckSeq) condition += n1cong*orthogCount;
    resp = Integer.parseInt(tokenizer.nextToken());
    rt = Integer.parseInt(tokenizer.nextToken());
    accuracy = Integer.parseInt(tokenizer.nextToken());
    if(isAllSeq) fullSequence();
    else if(isCueSwitch) cueSequence();
    boolean isValid = (!isTrimmed) ? true : trim();
    if(isValid){
     sumACC[condition] += accuracy;
     nACC[condition]++;
     if(accuracy==1){
      sumRT[condition] += rt;
      nRT[condition]++;
     }
    }
    if(multDist>0) n1dist = dist[0];
    n1targ = targ[0]; n1cong = cong;
    if(multCue>0) n1cue = cue[0];
    n1resp = resp;
    if(Main.nGoals>0) n1task = task;
    String nextTrial = in.readLine();
    if(nextTrial.equals("END_OF_FILE")) in = null;
    else tokenizer = new StringTokenizer(nextTrial);
   }
   catch(NumberFormatException | IOException e3){
    in = null;
    System.out.println("No END_OF_FILE in data file.");
   }
  }
  printToFile(i);
 }

 /**Runs a full CSE sequence analysis (15 conditions).*/
 private static void fullSequence(){
  if(n1cong==1){
   if(cong==1){
    condition = (n1targ==dist[0]) ? (n1dist==targ[0] ? 14 : 13) :
    (n1dist==targ[0] ? 12 : (n1targ==targ[0] ?
    (n1dist==dist[0] ? 11 : 10) : (n1dist==dist[0] ? 9 : 8)));
   }
   else condition = (n1targ==targ[0]) ? 7 : (n1dist==dist[0] ? 6 : 5);
  }
  else{
   if(cong==1)
    condition = (n1targ==targ[0]) ? 4 : (n1dist==dist[0] ? 3 : 2);
   else condition = (n1targ==targ[0]) ? 1 : 0;
  }
 }

 /**Runs a full task switch sequence analysis (10 conditions).*/
 private static void cueSequence(){
  if(n1cue==cue[0]){
   if(n1targ==targ[0]) condition = 0;
   else condition = (n1resp==resp) ? 1 : 2;
  }
  else if(n1task==task){
   if(n1targ==targ[0]) condition = 3;
   else condition = (n1resp==resp) ? 4 : 5;
  }
  else if(n1targ==targ[0]) condition = (n1resp==resp) ? 6 : 7;
  else condition = (n1resp==resp) ? 8 : 9;
 }

 /**Trims stimulus repetitions.
  * @return Kept or not?*/
 private static boolean trim(){
  if(n1targ!=targ[0] & n1dist!=dist[0]){
   if(Main.nConflicting>0)
    return (n1targ!=dist[0] & n1dist!=targ[0]) ? true : false;
   else return true;
  }
  else return false;
 }

 /**Prints means to file.
  * @param i Participant number*/
 private static void printToFile(int i){
  FileOutputStream out = null;
  PrintStream p = null;
  try{
   fileName = "out"+i+".txt";
   if(i>1) out = new FileOutputStream("Means.txt", true);
   else out = new FileOutputStream("Means.txt");
   p = new PrintStream(out);
  }
  catch(Exception e){
   System.out.println("Error setting output stream.");
  }
  for(int k=0; k<nConditions; k++){
   if(nRT[k]!=0){
    xRT[k] = sumRT[k]/nRT[k];
    p.print(xRT[k]+"\t");
   }
   else p.print("\t");   
  }
  for(int k=0; k<nConditions; k++){
   if(nACC[k]!=0){
    xERR[k] = (1-(sumACC[k]/nACC[k]))*100;
    p.print(xERR[k]+"\t");
   }
   else p.print("\t");   
  }
  p.println();
  try{
   p.close(); out.close();
  }
  catch(Exception e2){
   System.out.println("Error closing output stream");
  }
 }
}