/*
  ____    ___  ____                    
 |  _ \  / __)|  _ \                   
 | |_) )/ /_  | |_) )                  
 |  __/(   _) |  __/              _    
 | |    \ \__ | |     |\/| /\ |\ |_ |  
 |_|     \___)|_|     |  | \/ |/ |_ |_ 
                                       
*/

/**Parallel Episodic Processing (PEP) Model
 * @version 3.0.0
 * @author James R. Schmidt <james.schmidt@ugent.be>
 * Universiteit Gent*/

package pep;

import java.util.concurrent.Semaphore;

/**This is the Main class in which the model variables need to be
 * defined (in the VARIABLES section of the code).*/
public class Main{
 //----------VARIABLES------------------------------------------
 /**DataSet source*/
 static String source = "IBTRCE.txt";
 /**number of subjects*/              static final int nSubjects = 5000;
 /**number of concurrent threads*/    public static final int maxThreads = 5;
 //-------------------------------------------------------------
 /**random with replacement?*/        static boolean isReplace = true;
 /**number of blocks per subject*/    static int[] nBlocks = {};
 /**number of no replacement trials*/ static int[] nSamples = {16,1};
 /**two alternating stimulus sets?*/  static boolean twoSets = false;
 /**paired trial order?*/             static boolean isPaired = false;
 //-------------------------------------------------------------
 /**number of targets*/               static final int nTargets = 4;
 /**concurrent targets*/              static final int multTarg = 2;
 /**number of distracters*/           static final int nDistracters = 0;
 /**concurrent distracters*/          static final int multDist = 0;
 /**number of cues*/                  static final int nCues = 2;
 /**concurrent cues*/                 static final int multCue = 1;
 /**number of decisions*/             static final int nDecisions = 4;
 /**number of goals*/                 static final int nGoals = 2;
 /**number of responses*/             static final int nResponses = 2;
 /**number conflict stimuli*/         static final int nConflicting = 0;
 /**conditions per variable*/         static final int[] numPerVar = {3};
 //-------------------------------------------------------------
 /**target bias*/                     static final double tBias = 1;
 /**target signal*/                   static final double tSignal = 1;
 /**distracter bias*/                 static final double dBias = 1;
 /**distracter signal*/               static final double dSignal = 1;
 /**degraded signal*/                 static final double tDegrade = .65;
 /**degraded stimuli?*/               static final boolean isDegrade = false;
 //-------------------------------------------------------------
 /**target onset time*/               public static int tOnset = 1;
 /**target offset time*/              public static int tOffset = 2000;
 /**distracter onset time*/           public static int dOnset = 0;
 /**distracter offset time*/          public static int dOffset = 0;
 /**cue onset time*/                  public static int cOnset = 1;
 /**cue offset time*/                 public static int cOffset = 2000;
 /**distracter mask*/                 public static int dMask = 0;
 /**trial length*/                    public static int trialLength = 2000;
 /**interstimulus interval*/          public static int isi = 500;
 //-------------------------------------------------------------
 /**check sequence?*/        static boolean isCheckSeq = false;
 /**filter repetitions?*/    static boolean isTrimmed = false;
 /**check all rep types*/    static boolean isAllSeq = false;
 /**cued task switching?*/   static boolean isCueSwitch = false;
 //-------------------------------------------------------------
 /**overtrained target-decision connections*/
 static int[][] targDec = {{0,1,2,3},
                           {0,1,2,3}};
 /**overtrained distracter-decision connections*/
 static int[][] distDec = {{},
                           {}};
 /**overtrained decision-response connections*/
 static int[][] knownRes = {{},
                            {}};
 //-------------------------------------------------------------
 /**overtrained goal-decision links*/
 static int[][] goalDec = {{0,0,1,1},
                           {0,1,2,3}};
 /**overtrained goal-target links*/
 static int[][] goalTarg = {{0,0,1,1},
                            {0,1,2,3}};
 //-------------------------------------------------------------
 /**instructed cue-goal connections*/
 static int[][] iCueGoal = {{0,1},
                            {0,1}};
 /**instructed stimulus-decision-response connections*/
 static int[][] iStimRes = {{0,1,2,3,2,3,2,3},
                            {0,1,0,1,0,1,0,1}};
 /**instructed decision-response connections*/
 static int[][] iDecRes = {{},
                           {}};
 //-------------------------------------------------------------
 /**thread manager*/
 public static final Semaphore freeThread = new Semaphore(maxThreads);
 /**array of runs used for separate subject threads*/
 public static RunTime runs[] = new RunTime[nSubjects];
 /**give simulation feedback?*/ private static boolean isVerbose = true;

 /**For each participant, the main method calls RunTime (the guts of the
  * model), and then generates condition means with DataAnalysis and
  * creates a concatenated datafile with Concat.
  * @param args Not used.*/
 public static void main(String[] args) throws InterruptedException{
  for(int subj=0; subj<nSubjects; subj++){
   freeThread.acquire();
   runs[subj] = new RunTime(subj);
   Thread t = new Thread(runs[subj]);
   t.start();
   if(subj>19) runs[subj-20] = null; //free stack heap
   if(isVerbose)
    System.out.println(""+(subj+1)+" of "+nSubjects+" subjects started...");
  }

  freeThread.acquire(maxThreads);
  if(isVerbose) System.out.println("Run complete! Processing data...");

  DataAnalysis.main(new String[0]);
  if(isVerbose) System.out.println("Data analysis complete...");

  Concat.main(new String[0]);
  if(isVerbose) System.out.println("Data files concatenated...");
  freeThread.release(maxThreads);
 }
}